==> v8.16.5:x-pack.tar.gz: elasticsearch/plugin/core/src/test/java/org/elasticsearch/xpack/core/inference/results/RankedDocsResultsTests.java <==
/*
 * Copyright Elasticsearch B.V. and/or licensed to Elasticsearch B.V. under one
 * or more contributor license agreements. Licensed under the Elastic License
 * 2.0; you may not use this file except in compliance with the Elastic License
 * 2.0.
 */

package org.elasticsearch.xpack.core.inference.results;

import org.elasticsearch.TransportVersion;
import org.elasticsearch.common.io.stream.Writeable;
import org.elasticsearch.xcontent.XContentParser;
import org.elasticsearch.xpack.core.ml.AbstractChunkedBWCSerializationTestCase;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import static org.elasticsearch.TransportVersions.ML_RERANK_DOC_OPTIONAL;

public class RankedDocsResultsTests extends AbstractChunkedBWCSerializationTestCase<RankedDocsResults> {

    @Override
    protected Writeable.Reader<RankedDocsResults> instanceReader() {
        return RankedDocsResults::new;
    }

    @Override
    protected RankedDocsResults createTestInstance() {
        return createRandom();
    }

    public static RankedDocsResults createRandom() {
        return new RankedDocsResults(randomList(0, 10, RankedDocsResultsTests::createRandomDoc));
    }

    public static RankedDocsResults.RankedDoc createRandomDoc() {
        return new RankedDocsResults.RankedDoc(randomIntBetween(0, 100), randomFloat(), randomBoolean() ? null : randomAlphaOfLength(10));
    }

    @Override
    protected RankedDocsResults mutateInstance(RankedDocsResults instance) throws IOException {
        List<RankedDocsResults.RankedDoc> copy = new ArrayList<>(List.copyOf(instance.getRankedDocs()));
        copy.add(createRandomDoc());
        return new RankedDocsResults(copy);
    }

    @Override
    protected RankedDocsResults mutateInstanceForVersion(RankedDocsResults instance, TransportVersion fromVersion) {
        if (fromVersion.onOrAfter(ML_RERANK_DOC_OPTIONAL)) {
            return instance;
        } else {
            var compatibleDocs = rankedDocsNullStringToEmpty(instance.getRankedDocs());
            return new RankedDocsResults(compatibleDocs);
        }
    }

    private List<RankedDocsResults.RankedDoc> rankedDocsNullStringToEmpty(List<RankedDocsResults.RankedDoc> rankedDocs) {
        var result = new ArrayList<RankedDocsResults.RankedDoc>(rankedDocs.size());
        for (var doc : rankedDocs) {
            if (doc.text() == null) {
                result.add(new RankedDocsResults.RankedDoc(doc.index(), doc.relevanceScore(), ""));
            } else {
                result.add(doc);
            }
        }
        return result;
    }

    @Override
    protected RankedDocsResults doParseInstance(XContentParser parser) throws IOException {
        return RankedDocsResults.createParser(true).apply(parser, null);
    }
}

==> v8.17.0:x-pack.tar.gz: elasticsearch/plugin/core/src/test/java/org/elasticsearch/xpack/core/inference/results/RankedDocsResultsTests.java <==
/*
 * Copyright Elasticsearch B.V. and/or licensed to Elasticsearch B.V. under one
 * or more contributor license agreements. Licensed under the Elastic License
 * 2.0; you may not use this file except in compliance with the Elastic License
 * 2.0.
 */

package org.elasticsearch.xpack.core.inference.results;

import org.elasticsearch.TransportVersion;
import org.elasticsearch.TransportVersions;
import org.elasticsearch.common.io.stream.Writeable;
import org.elasticsearch.xcontent.XContentParser;
import org.elasticsearch.xpack.core.ml.AbstractChunkedBWCSerializationTestCase;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

public class RankedDocsResultsTests extends AbstractChunkedBWCSerializationTestCase<RankedDocsResults> {

    @Override
    protected Writeable.Reader<RankedDocsResults> instanceReader() {
        return RankedDocsResults::new;
    }

    @Override
    protected RankedDocsResults createTestInstance() {
        return createRandom();
    }

    public static RankedDocsResults createRandom() {
        return new RankedDocsResults(randomList(0, 10, RankedDocsResultsTests::createRandomDoc));
    }

    public static RankedDocsResults.RankedDoc createRandomDoc() {
        return new RankedDocsResults.RankedDoc(randomIntBetween(0, 100), randomFloat(), randomBoolean() ? null : randomAlphaOfLength(10));
    }

    @Override
    protected RankedDocsResults mutateInstance(RankedDocsResults instance) throws IOException {
        List<RankedDocsResults.RankedDoc> copy = new ArrayList<>(List.copyOf(instance.getRankedDocs()));
        copy.add(createRandomDoc());
        return new RankedDocsResults(copy);
    }

    @Override
    protected RankedDocsResults mutateInstanceForVersion(RankedDocsResults instance, TransportVersion fromVersion) {
        if (fromVersion.onOrAfter(TransportVersions.V_8_15_0)) {
            return instance;
        } else {
            var compatibleDocs = rankedDocsNullStringToEmpty(instance.getRankedDocs());
            return new RankedDocsResults(compatibleDocs);
        }
    }

    private List<RankedDocsResults.RankedDoc> rankedDocsNullStringToEmpty(List<RankedDocsResults.RankedDoc> rankedDocs) {
        var result = new ArrayList<RankedDocsResults.RankedDoc>(rankedDocs.size());
        for (var doc : rankedDocs) {
            if (doc.text() == null) {
                result.add(new RankedDocsResults.RankedDoc(doc.index(), doc.relevanceScore(), ""));
            } else {
                result.add(doc);
            }
        }
        return result;
    }

    @Override
    protected RankedDocsResults doParseInstance(XContentParser parser) throws IOException {
        return RankedDocsResults.createParser(true).apply(parser, null);
    }
}
